# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>
import bpy
from bpy.types import Header, Menu, Panel
from rna_prop_ui import PropertyPanel
from bl_ui.properties_grease_pencil_common import GreasePencilDataPanel, GreasePencilToolsPanel
from bpy.app.translations import pgettext_iface as iface_

strip_types = {
    'MOVIE' : 'Movie',
    'SOUND' : 'Sound',
    'IMAGE' : 'Image',
    'META' : 'Meta',
    'SCENE' : 'Scene',
    'MULTICAM' : 'Multicam Selector',
    'TRANSFORM' : 'Transform',
    'COLOR' : 'Color',
    'SPEED' : 'Speed Control',
    'ADJUSTMENT' : 'Adjustment Layer',
    'CROSS' : 'Cross',
    'GAMMA_CROSS' : 'Gamma Cross',
    'ALPHA_OVER' : 'Alpha Over',
    'ALPHA_UNDER' : 'Alpha Under',
    'OVER_DROP' : 'Alpha Over Drop',
    'GLOW' : 'Glow',
    'ADD' : 'Add',
    'SUBTRACT' : 'Subtract',
    'MULTIPLY' : 'Multiply',
    'WIPE' : 'Wipe',
    'MASK' : 'Mask',
    'MOVIECLIP' : 'Clip',
    'GAUSSIAN_BLUR' : 'Gaussian Blur',
    'TEXT': 'Text'
}

def act_strip(context):
    try:
        return context.scene.sequence_editor.active_strip
    except AttributeError:
        return None


def draw_color_balance(layout, color_balance):
    
    row = layout.row()
    
    col = row.column()
    col.label(text="Lift:")
    col.template_color_picker(color_balance, "lift", value_slider=True, lock=True, lock_luminosity=True, cubic=True)
    sub = col.row(align=True)
    sub.prop(color_balance, "lift", text="")
    sub.prop(color_balance, "invert_lift", text="", icon_only=True, icon="DISCLOSURE_TRI_DOWN")
    
    col = row.column()
    col.label(text="Gamma:")
    col.template_color_picker(color_balance, "gamma", value_slider=True, lock=True, lock_luminosity=True, cubic=True)
    sub = col.row(align=True)
    sub.prop(color_balance, "gamma", text="")
    sub.prop(color_balance, "invert_gamma", text="", icon_only=True, icon="DISCLOSURE_TRI_DOWN")
    
    col = row.column()
    col.label(text="Gain:")
    col.template_color_picker(color_balance, "gain", value_slider=True, lock=True, lock_luminosity=True, cubic=True)
    sub = col.row(align=True)
    sub.prop(color_balance, "gain", text="")
    sub.prop(color_balance, "invert_gain", text="", icon_only=True, icon="DISCLOSURE_TRI_DOWN")
    


class SEQUENCER_HT_header(Header):
    bl_space_type = 'SEQUENCE_EDITOR'

    def draw(self, context):
        layout = self.layout

        st = context.space_data
        scene = context.scene

        row = layout.row(align=True)
        row.template_header()

        SEQUENCER_MT_editor_menus.draw_collapsible(context, layout)

        row = layout.row(align=True)
        row.prop(scene, "use_preview_range", text="", toggle=True)
        row.prop(scene, "lock_frame_selection_to_range", text="", toggle=True)

        layout.prop(st, "view_type", expand=True, text="")

        if st.view_type in {'PREVIEW', 'SEQUENCER_PREVIEW'}:
            layout.prop(st, "display_mode", expand=True, text="")

        if st.view_type == 'SEQUENCER':
            row = layout.row(align=True)
            row.operator("sequencer.copy", text="", icon='COPYDOWN')
            row.operator("sequencer.paste", text="", icon='PASTEDOWN')

            layout.separator()
            layout.operator("sequencer.refresh_all")
            layout.prop(st, "show_backdrop")
        else:
            if st.view_type == 'SEQUENCER_PREVIEW':
                layout.separator()
                layout.operator("sequencer.refresh_all")

            layout.prop(st, "preview_channels", expand=True, text="")
            layout.prop(st, "display_channel", text="Channel")

            ed = context.scene.sequence_editor
            if ed:
                row = layout.row(align=True)
                row.prop(ed, "show_overlay", text="", icon='GHOST_ENABLED')
                if ed.show_overlay:
                    row.prop(ed, "overlay_frame", text="")
                    row.prop(ed, "use_overlay_lock", text="", icon='LOCKED')

                    row = layout.row()
                    row.prop(st, "overlay_type", text="")

        if st.view_type in {'PREVIEW', 'SEQUENCER_PREVIEW'}:
            gpd = context.gpencil_data
            toolsettings = context.tool_settings

            # Proportional editing
            if gpd and gpd.use_stroke_edit_mode:
                row = layout.row(align=True)
                row.prop(toolsettings, "proportional_edit", icon_only=True)
                if toolsettings.proportional_edit != 'DISABLED':
                    row.prop(toolsettings, "proportional_edit_falloff", icon_only=True)

        row = layout.row(align=True)
        row.operator("render.opengl", text="", icon='RENDER_STILL').sequencer = True
        props = row.operator("render.opengl", text="", icon='RENDER_ANIMATION')
        props.animation = True
        props.sequencer = True

        layout.template_running_jobs()


class SEQUENCER_MT_editor_menus(Menu):
    bl_idname = "SEQUENCER_MT_editor_menus"
    bl_label = ""

    def draw(self, context):
        self.draw_menus(self.layout, context)

    @staticmethod
    def draw_menus(layout, context):
        st = context.space_data

        layout.menu("SEQUENCER_MT_view")

        if st.view_type in {'SEQUENCER', 'SEQUENCER_PREVIEW'}:
            layout.menu("SEQUENCER_MT_select")
            layout.menu("SEQUENCER_MT_marker")
            layout.menu("SEQUENCER_MT_add")
            layout.menu("SEQUENCER_MT_frame")
            layout.menu("SEQUENCER_MT_strip")


class SEQUENCER_MT_view_toggle(Menu):
    bl_label = "View Type"

    def draw(self, context):
        layout = self.layout

        layout.operator("sequencer.view_toggle").type = 'SEQUENCER'
        layout.operator("sequencer.view_toggle").type = 'PREVIEW'
        layout.operator("sequencer.view_toggle").type = 'SEQUENCER_PREVIEW'


class SEQUENCER_MT_view(Menu):
    bl_label = "View"

    def draw(self, context):
        layout = self.layout

        st = context.space_data
        is_preview = st.view_type in {'PREVIEW', 'SEQUENCER_PREVIEW'}
        is_sequencer_view = st.view_type in {'SEQUENCER', 'SEQUENCER_PREVIEW'}

        if st.view_type == 'PREVIEW':
            # Specifying the REGION_PREVIEW context is needed in preview-only
            # mode, else the lookup for the shortcut will fail in
            # wm_keymap_item_find_props() (see #32595).
            layout.operator_context = 'INVOKE_REGION_PREVIEW'
        layout.operator("sequencer.properties", icon='MENU_PANEL')
        layout.operator_context = 'INVOKE_DEFAULT'

        layout.separator()

        if is_sequencer_view:
            layout.operator_context = 'INVOKE_REGION_WIN'
            layout.operator("sequencer.view_all", text="View all Sequences")
            layout.operator("sequencer.view_selected")
            layout.operator_context = 'INVOKE_DEFAULT'
        if is_preview:
            layout.operator_context = 'INVOKE_REGION_PREVIEW'
            layout.operator("sequencer.view_all_preview", text="Fit preview in window")

            layout.separator()

            ratios = ((1, 8), (1, 4), (1, 2), (1, 1), (2, 1), (4, 1), (8, 1))

            for a, b in ratios:
                layout.operator("sequencer.view_zoom_ratio", text=iface_("Zoom %d:%d") % (a, b), translate=False).ratio = a / b

            layout.separator()

            layout.operator_context = 'INVOKE_DEFAULT'

            # # XXX, invokes in the header view
            # layout.operator("sequencer.view_ghost_border", text="Overlay Border")

        if is_sequencer_view:
            layout.prop(st, "show_seconds")
            layout.prop(st, "show_frame_indicator")
            layout.prop(st, "show_strip_offset")

            layout.prop_menu_enum(st, "waveform_draw_type")

        if is_preview:
            if st.display_mode == 'IMAGE':
                layout.prop(st, "show_safe_areas")
                layout.prop(st, "show_metadata")
            elif st.display_mode == 'WAVEFORM':
                layout.prop(st, "show_separate_color")

        layout.separator()

        if is_sequencer_view:
            layout.prop(st, "use_marker_sync")
            layout.separator()

        layout.operator("screen.area_dupli")
        layout.operator("screen.screen_full_area", text="Toggle Maximize Area")
        layout.operator("screen.screen_full_area").use_hide_panels = True


class SEQUENCER_MT_select(Menu):
    bl_label = "Select"

    def draw(self, context):
        layout = self.layout

        layout.operator("sequencer.select_active_side", text="Strips to the Left").side = 'LEFT'
        layout.operator("sequencer.select_active_side", text="Strips to the Right").side = 'RIGHT'
        props = layout.operator("sequencer.select", text="All strips to the Left")
        props.left_right = 'LEFT'
        props.linked_time = True
        props = layout.operator("sequencer.select", text="All strips to the Right")
        props.left_right = 'RIGHT'
        props.linked_time = True

        layout.separator()
        layout.operator("sequencer.select_handles", text="Surrounding Handles").side = 'BOTH'
        layout.operator("sequencer.select_handles", text="Left Handle").side = 'LEFT'
        layout.operator("sequencer.select_handles", text="Right Handle").side = 'RIGHT'
        layout.separator()
        layout.operator_menu_enum("sequencer.select_grouped", "type", text="Grouped")
        layout.operator("sequencer.select_linked")
        layout.operator("sequencer.select_less")
        layout.operator("sequencer.select_more")
        layout.operator("sequencer.select_all").action = 'TOGGLE'
        layout.operator("sequencer.select_all", text="Inverse").action = 'INVERT'


class SEQUENCER_MT_marker(Menu):
    bl_label = "Marker"

    def draw(self, context):
        layout = self.layout

        from bl_ui.space_time import marker_menu_generic
        marker_menu_generic(layout)


class SEQUENCER_MT_change(Menu):
    bl_label = "Change"

    def draw(self, context):
        layout = self.layout
        strip = act_strip(context)

        layout.operator_context = 'INVOKE_REGION_WIN'

        layout.operator_menu_enum("sequencer.change_effect_input", "swap")
        layout.operator_menu_enum("sequencer.change_effect_type", "type")
        prop = layout.operator("sequencer.change_path", text="Path/Files")

        if strip:
            stype = strip.type

            if stype == 'IMAGE':
                prop.filter_image = True
            elif stype == 'MOVIE':
                prop.filter_movie = True
            elif stype == 'SOUND':
                prop.filter_sound = True


class SEQUENCER_MT_frame(Menu):
    bl_label = "Frame"

    def draw(self, context):
        layout = self.layout

        layout.operator("anim.previewrange_clear")
        layout.operator("anim.previewrange_set")


class SEQUENCER_MT_add(Menu):
    bl_label = "Add"

    def draw(self, context):
        layout = self.layout

        layout.operator_context = 'INVOKE_REGION_WIN'

        if len(bpy.data.scenes) > 10:
            layout.operator_context = 'INVOKE_DEFAULT'
            layout.operator("sequencer.scene_strip_add", text="Scene...")
        else:
            layout.operator_menu_enum("sequencer.scene_strip_add", "scene", text="Scene...")

        if len(bpy.data.movieclips) > 10:
            layout.operator_context = 'INVOKE_DEFAULT'
            layout.operator("sequencer.movieclip_strip_add", text="Clips...")
        else:
            layout.operator_menu_enum("sequencer.movieclip_strip_add", "clip", text="Clip...")

        if len(bpy.data.masks) > 10:
            layout.operator_context = 'INVOKE_DEFAULT'
            layout.operator("sequencer.mask_strip_add", text="Masks...")
        else:
            layout.operator_menu_enum("sequencer.mask_strip_add", "mask", text="Mask...")

        layout.operator("sequencer.movie_strip_add", text="Movie")
        layout.operator("sequencer.image_strip_add", text="Image")
        layout.operator("sequencer.sound_strip_add", text="Sound")

        layout.menu("SEQUENCER_MT_add_effect")


class SEQUENCER_MT_add_effect(Menu):
    bl_label = "Effect Strip..."

    def draw(self, context):
        layout = self.layout

        layout.operator_context = 'INVOKE_REGION_WIN'

        layout.operator("sequencer.effect_strip_add", text="Add").type = 'ADD'
        layout.operator("sequencer.effect_strip_add", text="Subtract").type = 'SUBTRACT'
        layout.operator("sequencer.effect_strip_add", text="Alpha Over").type = 'ALPHA_OVER'
        layout.operator("sequencer.effect_strip_add", text="Alpha Under").type = 'ALPHA_UNDER'
        layout.operator("sequencer.effect_strip_add", text="Cross").type = 'CROSS'
        layout.operator("sequencer.effect_strip_add", text="Gamma Cross").type = 'GAMMA_CROSS'
        layout.operator("sequencer.effect_strip_add", text="Gaussian Blur").type = 'GAUSSIAN_BLUR'
        layout.operator("sequencer.effect_strip_add", text="Multiply").type = 'MULTIPLY'
        layout.operator("sequencer.effect_strip_add", text="Over Drop").type = 'OVER_DROP'
        layout.operator("sequencer.effect_strip_add", text="Wipe").type = 'WIPE'
        layout.operator("sequencer.effect_strip_add", text="Glow").type = 'GLOW'
        layout.operator("sequencer.effect_strip_add", text="Text").type = 'TEXT'
        layout.operator("sequencer.effect_strip_add", text="Transform").type = 'TRANSFORM'
        layout.operator("sequencer.effect_strip_add", text="Color").type = 'COLOR'
        layout.operator("sequencer.effect_strip_add", text="Speed Control").type = 'SPEED'
        layout.operator("sequencer.effect_strip_add", text="Multicam Selector").type = 'MULTICAM'
        layout.operator("sequencer.effect_strip_add", text="Adjustment Layer").type = 'ADJUSTMENT'


class SEQUENCER_MT_strip(Menu):
    bl_label = "Strip"

    def draw(self, context):
        layout = self.layout

        layout.operator_context = 'INVOKE_REGION_WIN'

        layout.operator("transform.transform", text="Grab/Move").mode = 'TRANSLATION'
        layout.operator("transform.transform", text="Grab/Extend from frame").mode = 'TIME_EXTEND'
        layout.operator("sequencer.gap_remove").all = False
        layout.operator("sequencer.gap_insert")

        #  uiItemO(layout, NULL, 0, "sequencer.strip_snap"); // TODO - add this operator
        layout.separator()

        layout.operator("sequencer.cut", text="Cut (hard) at frame").type = 'HARD'
        layout.operator("sequencer.cut", text="Cut (soft) at frame").type = 'SOFT'
        layout.operator("sequencer.slip", text="Slip Strip Contents")
        layout.operator("sequencer.images_separate")
        layout.operator("sequencer.offset_clear")
        layout.operator("sequencer.deinterlace_selected_movies")
        layout.operator("sequencer.rebuild_proxy")
        layout.separator()

        layout.operator("sequencer.duplicate_move")
        layout.operator("sequencer.delete")

        strip = act_strip(context)

        if strip:
            stype = strip.type

            # XXX note strip.type is never equal to 'EFFECT', look at seq_type_items within rna_sequencer.c
            if stype == 'EFFECT':
                pass
                # layout.separator()
                # layout.operator("sequencer.effect_change")
                # layout.operator("sequencer.effect_reassign_inputs")
            elif stype == 'IMAGE':
                layout.separator()
                # layout.operator("sequencer.image_change")
                layout.operator("sequencer.rendersize")
            elif stype == 'SCENE':
                pass
                # layout.separator()
                # layout.operator("sequencer.scene_change", text="Change Scene")
            elif stype == 'MOVIE':
                layout.separator()
                # layout.operator("sequencer.movie_change")
                layout.operator("sequencer.rendersize")
            elif stype == 'SOUND':
                layout.separator()
                layout.operator("sequencer.crossfade_sounds")

        layout.separator()

        layout.operator("sequencer.meta_make")
        layout.operator("sequencer.meta_separate")

        #if (ed && (ed->metastack.first || (ed->act_seq && ed->act_seq->type == SEQ_META))) {
        #	uiItemS(layout);
        #	uiItemO(layout, NULL, 0, "sequencer.meta_toggle");
        #}

        layout.separator()
        layout.operator("sequencer.reload", text="Reload Strips")
        layout.operator("sequencer.reload", text="Reload Strips and Adjust Length").adjust_length = True
        layout.operator("sequencer.reassign_inputs")
        layout.operator("sequencer.swap_inputs")

        layout.separator()
        layout.operator("sequencer.lock")
        layout.operator("sequencer.unlock")
        layout.operator("sequencer.mute").unselected = False
        layout.operator("sequencer.unmute").unselected = False

        layout.operator("sequencer.mute", text="Mute Deselected Strips").unselected = True

        layout.operator("sequencer.snap")

        layout.operator_menu_enum("sequencer.swap", "side")

        layout.separator()

        layout.operator("sequencer.swap_data")
        layout.menu("SEQUENCER_MT_change")


class SequencerButtonsPanel:
    bl_space_type = 'SEQUENCE_EDITOR'
    bl_region_type = 'UI'

    @staticmethod
    def has_sequencer(context):
        return (context.space_data.view_type in {'SEQUENCER', 'SEQUENCER_PREVIEW'})

    @classmethod
    def poll(cls, context):
        return cls.has_sequencer(context) and (act_strip(context) is not None)


class SequencerButtonsPanel_Output:
    bl_space_type = 'SEQUENCE_EDITOR'
    bl_region_type = 'UI'

    @staticmethod
    def has_preview(context):
        st = context.space_data
        return (st.view_type in {'PREVIEW', 'SEQUENCER_PREVIEW'}) or st.show_backdrop

    @classmethod
    def poll(cls, context):
        return cls.has_preview(context)


class SEQUENCER_PT_edit(SequencerButtonsPanel, Panel):
    bl_label = "Strip"
        
    def draw(self, context):
        layout = self.layout

        scene = context.scene
        frame_current = scene.frame_current
        strip = act_strip(context)


        row = layout.row(align=True)
        row.prop(strip, "name", text="",)
        row.prop(strip, "lock", toggle=True, icon_only=True)
        
        row = layout.row(align=True)
        sub = row.row(align=True)
        sub.active = (not strip.mute)
        
        if not (strip.type == 'SOUND'):
            row.prop(strip, "mute", toggle=True, icon_only=True)
            sub.prop(strip, "blend_type", text="")
            sub.prop(strip, "blend_alpha", text="Opacity", slider=True)
        else:
            row.prop(strip, "mute", toggle=True, icon_only=True, icon="MUTE_IPO_OFF")
            sub.prop(strip, "volume", text="Volume", slider=True)
            row = layout.row(align=False)
            row.active = not strip.mute
            row.prop(strip, "pan", text="Pan", slider=True)
            row.prop(strip, "pitch", text="Pitch", slider=True)
            
        if strip.type == "SCENE":
            row = layout.row(align=True)
            row.enabled = not strip.mute
            row.prop(strip.scene, "audio_volume", text="Volume", slider=True)
        
        col = layout.column()
        col.enabled = not strip.lock
        
        #col.separator()
        row = col.row()
        
        sub = row.column(align=True)
        sub.prop(strip, "frame_start", text="Start")
        sub.prop(strip, "frame_final_end", text="End")

        sub = row.column(align=True)
        sub.prop(strip, "channel")
        sub.prop(strip, "frame_final_duration")

        
        if strip.type in [ 'MOVIE', 'IMAGE', 'SOUND', 'META', 'SCENE', 'MASK', 'MOVIECLIP' ]:
            split = layout.split(align=False)
            split.enabled = not strip.lock
            
            col = split.column(align=True)
            col.label(text="Trim (hard):")
            col.prop(strip, "animation_offset_start", text="Start")
            col.prop(strip, "animation_offset_end", text="End")

            col = split.column(align=True)
            col.label(text="Trim (soft):")
            col.prop(strip, "frame_offset_start", text="Start")
            col.prop(strip, "frame_offset_end", text="End")


class SEQUENCER_PT_info(SequencerButtonsPanel, Panel):
    bl_label = "Info"
    bl_options = {'DEFAULT_CLOSED'}
     
    def draw(self, context):
        layout = self.layout

        scene = context.scene
        frame_current = scene.frame_current
        strip = act_strip(context)
        
        col = layout.column(align=True)
        
        col.label(text=iface_("Type: %s") % strip_types[strip.type], translate=False)
        
        col.label(text=iface_("Begin:       %s   %d") % (bpy.utils.smpte_from_frame(strip.frame_start), 
                  strip.frame_start), translate=False)
                  
        col.label(text=iface_("End:          %s   %d") % (bpy.utils.smpte_from_frame(strip.frame_final_end), 
                  strip.frame_final_end), translate=False)
        
        col.label(text=iface_("Duration:  %s   %d") % (bpy.utils.smpte_from_frame(strip.frame_final_duration), 
                  strip.frame_final_duration), translate=False)

        playhead = frame_current - strip.frame_start
        sub = col.column()
        sub.active = (frame_current >= strip.frame_start and frame_current <= strip.frame_start + strip.frame_final_duration)
        sub.label(text=iface_("Playhead:  %s   %d") % (bpy.utils.smpte_from_frame(playhead), playhead), translate=False)
        
        
class SEQUENCER_PT_effect(SequencerButtonsPanel, Panel):
    bl_label = "Effect"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return strip.type in {'ADD', 'SUBTRACT', 'ALPHA_OVER', 'ALPHA_UNDER',
                              'CROSS', 'GAMMA_CROSS', 'MULTIPLY', 'OVER_DROP',
                              'WIPE', 'GLOW', 'TRANSFORM', 'COLOR', 'SPEED',
                              'MULTICAM', 'GAUSSIAN_BLUR', 'TEXT'}

    def draw(self, context):
        layout = self.layout

        strip = act_strip(context)
        sequencer = context.scene.sequence_editor
        
        # Draw Type option in blend effects
        if strip.input_count > 1:
            split = layout.split(percentage=0.25)
            split.label(text="Type:")
            split.operator_menu_enum("sequencer.change_effect_type", "type", text=strip_types[strip.type])
          
        if strip.input_count > 0:
            
            split = layout.split(percentage=0.25)
            
            if strip.input_count == 1:
                split.label(text="Input:")
            else:
                split.label(text="Input 1:")
            
            sequences_object = sequencer
            if sequencer.meta_stack:
                sequences_object = sequencer.meta_stack[-1]
            split.prop_search(strip, "input_1", sequences_object, "sequences", text="")
            
            # Bug Alert: This will need to be removed  
            if (strip.input_1 and strip.input_1.name == strip.name):
                sub = layout.column(align=True)
                sub.label(text="Error: Input is the own strip")
                sub.label(text="Blender will crash!")
            
            if strip.input_count > 1: 
                
                # 2 inputs effects
                
                split = layout.split(percentage=0.25)
                split.label(text="Input 2:")
                split.prop_search(strip, "input_2", sequences_object, "sequences", text="")
                
                # Bug Alert: This will need to be removed  
                if (strip.input_2 and strip.input_2.name == strip.name):
                    sub = layout.column(align=True)
                    sub.label(text="Error: Input is the own strip")
                    sub.label(text="Blender will crash!")
            
                split = layout.split(percentage=0.25)
                split.label(text="Inputs:")
                row = split.row()
                row.operator("sequencer.swap_inputs", text="Swap")
                row.operator("sequencer.reassign_inputs", text="Reassign")
                
                split = layout.split(percentage=0.25)
                split.label(text="Fade:")
                row = split.row(align=True)
                sub = row.row(align=True)
                sub.enabled = not strip.use_default_fade
                sub.prop(strip, "effect_fader", text="")
                sub = row.row(align=True)
                sub.scale_x = 0.4
                sub.prop(strip, "use_default_fade", text="Auto", toggle=True)
             
             
                if strip.type == 'WIPE':
                    col = layout.column()
                    col.label(text="Transition:")
                    row = col.row()
                    row.prop(strip, "transition_type", expand=True)
                    split = layout.split(percentage=0.33)
                    split.label(text="Direction:")
                    row = split.row()
                    row.prop(strip, "direction", expand=True)

                    row = layout.row()
                    col = row.column()
                    col.prop(strip, "blur_width", slider=True)
                    if strip.transition_type in {'SINGLE', 'DOUBLE'}:
                        col = row.column()
                        col.prop(strip, "angle")
                        
            else: 
                
                # 1 input Effects
                
                if strip.type == 'GLOW':
                    row = layout.row()
                    col = row.column(align=True)
                    col.prop(strip, "threshold", slider=True)
                    col.prop(strip, "clamp", slider=True)
                    
                    col = row.column(align=True)
                    col.prop(strip, "boost_factor")
                    col.prop(strip, "blur_radius", text="Radius")

                    row = layout.row()
                    row.prop(strip, "quality", slider=True)
                    row.prop(strip, "use_only_boost")


                elif strip.type == 'SPEED':
                    split =layout.split(percentage=0.25)
                    
                    label = "Speed:"
                    if not strip.use_default_fade:
                        if not strip.use_as_speed:
                            label = "Frame:"
                        if strip.scale_to_length and not strip.use_as_speed:
                            label = "Scale:"
                            
                    split.label(text=label)
                    row = split.row(align=True)
                    sub = row.row(align=True)
                    sub.enabled = not strip.use_default_fade
                    sub.prop(strip, "speed_factor", text="")
                    sub = row.row(align=True)
                    sub.scale_x = 0.4
                    sub.prop(strip, "use_default_fade", text="Auto", toggle=True)
                    
                    row = layout.row()
                    col = row.column()
                    sub = col.row()
                    sub.enabled = not strip.use_default_fade and not strip.scale_to_length
                    sub.prop(strip, "use_as_speed", text="Use as speed")
                    sub = col.row()
                    sub.enabled = not strip.use_default_fade and not strip.use_as_speed
                    sub.prop(strip, "scale_to_length", text="Use as scale")
                    
                    col = row.column()
                    col.label(text="Multiply:")
                    col.prop(strip, "multiply_speed", text="")
                    

                elif strip.type == 'TRANSFORM':
                    
                    row = layout.row(align=False)
                    col = row.column(align=True)
                    col.label(text="Position:")
                    col.prop(strip, "translate_start_x", text="X")
                    col.prop(strip, "translate_start_y", text="Y")
                    sub = col.row(align=True)
                    sub.scale_x = 0.5
                    sub.prop(strip, "translation_unit", expand=True)
                    
                    col = row.column(align=True)
                    col.label(text="Scale:")
                    col.prop(strip, "scale_start_x", text="X")
                    if (not strip.use_uniform_scale):
                        col.prop(strip, "scale_start_y", text="Y")
                    else:
                        sub = col.row(align=True)
                        sub.enabled = False
                        sub.prop(strip, "scale_start_x", text="Y")
                    col.prop(strip, "use_uniform_scale")
                    
                    row = layout.row(align=False)
                    col = row.column(align=True)
                    col.label(text="Rotation:")
                    col.prop(strip, "rotation_start", text="")
                    
                    col = row.column(align=True)
                    col.label(text="Interpolation:")
                    col.prop(strip, "interpolation", text="")
                
                
                if strip.type == 'GAUSSIAN_BLUR':
                    split = layout.split(percentage=0.25)
                    split.label("Size:")
                    row= split.row(align=True)
                    row.prop(strip, "size_x", text="X")
                    row.prop(strip, "size_y", text="Y")
                 
        else: 
            
            # 0 inputs Effects
            
            if strip.type == 'COLOR':
                layout.prop(strip, "color")
                layout.template_color_picker(strip, "color", value_slider=True, cubic=True)


            elif strip.type == 'MULTICAM':
                split = layout.split(percentage=0.5)
            
                split.prop(strip, "multicam_source", text="Channel")
                
                row = split.row(align=True)
                
                bpy_ops_seq = bpy.ops.sequencer
                if bpy_ops_seq and bpy_ops_seq.jumpnext:
                    row.scale_x = 3.0
                    row.operator("sequencer.jumpprev", text="", icon='REW')
                row.operator("screen.animation_play", text="", icon='PAUSE' if context.screen.is_animation_playing else 'PLAY')
                if bpy_ops_seq and bpy_ops_seq.jumpnext:
                    row.operator("sequencer.jumpnext", text="", icon='FF')

                split = layout.split(percentage=0.25)
                col = split.column(align=True)
                row = col.row(align=True)
                
                strip_channel = strip.channel
                
                if (strip_channel > 2 ):
                    row.label("Cut To:")
                    col = split.column(align=True)
                    
                    BT_ROW = 4
                    
                    for i in range(1, strip_channel):
                        
                        if (i % BT_ROW == 1):
                            row = col.row(align=True)
                            
                        if (i == strip.multicam_source):
                            sub = row.row(align=True)
                            sub.enabled = False
                            sub.operator("sequencer.cut_multicam", text="[%d]" % i).camera = i
                        else:
                            row.operator("sequencer.cut_multicam", text="%d" % i).camera = i
                    
                    if ((strip.channel > BT_ROW) and ((strip_channel - 1) % BT_ROW)):
                        sub = row.row(align=True)
                        sub.enabled = False
                    
                        for i in range(strip.channel, strip_channel + ((BT_ROW + 1 - strip_channel) % BT_ROW)):
                            sub.operator("sequencer.cut_multicam", text="" )

            elif strip.type == 'TEXT':
                layout.prop(strip, "text", text="", toggle=True)
                
                row = layout.row()
                col = row.column()
                col.prop(strip, "location")
                col = row.column(align=True)
                col.prop(strip, "use_shadow")
                col.prop(strip, "font_size")
                col.prop(strip, "wrap_width", text="Wrap")
                
                col = layout.column(align=False)
                col.label("Alignment:")
                row = col.row()
                row.label(text="Horizontal")
                row.prop(strip, "align_x", expand=True)
                row = col.row()
                row.label(text="Vertical")
                row.prop(strip, "align_y", expand=True)
                
                layout.operator("sequencer.export_subtitles")



class SEQUENCER_PT_input(SequencerButtonsPanel, Panel):
    bl_label = "Input"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return strip.type in {'MOVIE', 'IMAGE', 'SCENE', 'MOVIECLIP', 'META',
                              'ADD', 'SUBTRACT', 'ALPHA_OVER', 'ALPHA_UNDER',
                              'CROSS', 'GAMMA_CROSS', 'MULTIPLY', 'OVER_DROP',
                              'WIPE', 'GLOW', 'TRANSFORM', 'COLOR', 'TEXT', 'MASK',
                              'MULTICAM', 'SPEED', 'ADJUSTMENT', 'GAUSSIAN_BLUR'}

    def draw(self, context):
        layout = self.layout
        scene = context.scene

        strip = act_strip(context)

        
        seq_type = strip.type

        # draw a filename if we have one
        if seq_type == 'IMAGE':
            col = layout.column(align=True)
            col.prop(strip, "directory", text="", icon="FILE_FOLDER")

            # Current element for the filename

            elem = strip.strip_elem_from_frame(scene.frame_current)
            if elem:
                row = col.row(align=True)
                row.prop(elem, "filename", text="", icon="FILE_IMAGE")  # strip.elements[0] could be a fallback
                row.operator("sequencer.change_path", text="", icon="FILESEL").filter_image=True

            if elem and elem.orig_width > 0 and elem.orig_height > 0:
                layout.label(text=iface_("Resolution: %dx%d, Frames: %d") % (elem.orig_width, elem.orig_height, strip.frame_duration), translate=False)
            else:
                layout.label(text="File not loaded")

            row = layout.row()
           
            col = row.column()
            col.label(text="Color space:")
            col.prop(strip.colorspace_settings, "name", text="" )
            
            col = row.column()
            col.label(text="Alpha Mode:")
            col.prop(strip, "alpha_mode", text="")


        elif seq_type == 'MOVIE':
            
            col = layout.column(align=True)
            col.prop(strip, "filepath", text="", icon="FILE_FOLDER")
            
            # Current element for info            
            elem = strip.elements[0]
            if elem:
                row = col.row(align=True)
                row.prop(elem, "filename", text="", icon="FILE_MOVIE")  # strip.elements[0] could be a fallback
                row.operator("sequencer.change_path", text="", icon="FILESEL").filter_movie=True

            if elem and elem.orig_width > 0 and elem.orig_height > 0:
                layout.label(text=iface_("Resolution: %dx%d, Frames: %d") % (elem.orig_width, elem.orig_height, strip.frame_duration), translate=False)
            else:
                layout.label(text="File not loaded")
            
            split = layout.split(percentage=0.5)
           
            col = split.column(align=True)
            col.label(text="Color Space:")
            col.prop(strip.colorspace_settings, "name", text="" )
            
            col = split.column(align=True)
            col.prop(strip, "stream_index", text="Stream")
            col.prop(strip, "mpeg_preseek", text="Preseek")
            
            
        elif seq_type == "META":
            split = layout.split(percentage=0.25)
            split.label("Meta:")
            row = split.row()
            row.operator("sequencer.meta_toggle", text="Edit")
            row.operator("sequencer.meta_separate", text="Undo")
          

        split = layout.split(percentage=0.5)
        col = split.column()
        col.prop(strip, "use_flip_x", text="Flip X")
        col.prop(strip, "use_translation", text="Position")
        
              
        col = split.column()
        col.prop(strip, "use_flip_y", text="Flip Y")
        col.prop(strip, "use_crop", text="Crop")
        
        split = layout.split(percentage=0.34)
        col = split.column()
        
        if strip.use_translation:
            sub = col.column(align=True)
            sub.prop(strip.transform, "offset_x", text="X")
            sub.prop(strip.transform, "offset_y", text="Y")
        
        col = split.column()
        if strip.use_crop:
            sub = col.column(align=True)
            subrow = sub.row(align=True)
            subrow.prop(strip.crop, "max_y")
            subrow.prop(strip.crop, "min_y")
            subrow = sub.row(align=True)
            subrow.prop(strip.crop, "min_x")
            subrow.prop(strip.crop, "max_x")

        if scene.render.use_multiview and seq_type in {'IMAGE', 'MOVIE'}:
            layout.prop(strip, "use_multiview")

            col = layout.column()
            col.active = strip.use_multiview

            col.label(text="Views Format:")
            col.row().prop(strip, "views_format", expand=True)

            box = col.box()
            box.active = strip.views_format == 'STEREO_3D'
            box.template_image_stereo_3d(strip.stereo_3d_format)


class SEQUENCER_PT_sound(SequencerButtonsPanel, Panel):
    bl_label = "Sound"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return (strip.type == 'SOUND')

    def draw(self, context):
        layout = self.layout

        st = context.space_data
        strip = act_strip(context)
        sound = strip.sound

        layout.template_ID(strip, "sound", open="sound.open")
        
        if sound is not None:
            row = layout.row(align=True)
            if sound.packed_file:
                row.operator("sound.unpack", icon='PACKAGE', text="")
            else:
                row.operator("sound.pack", icon='UGLYPACKAGE', text="")
            row.prop(sound, "filepath", text="")
            
            col = layout.column(align=True)
            row = col.row(align=False)
            row.prop(sound, "use_memory_cache", text="Cache")

            row.prop(sound, "use_mono")

        if st.waveform_draw_type == 'DEFAULT_WAVEFORMS':
            col.prop(strip, "show_waveform")


class SEQUENCER_PT_scene(SequencerButtonsPanel, Panel):
    bl_label = "Scene"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return (strip.type == 'SCENE')

    def draw(self, context):
        layout = self.layout

        strip = act_strip(context)

        layout.template_ID(strip, "scene")
        scene = strip.scene
        
        if scene:
            sta = scene.frame_start
            end = scene.frame_end
            layout.label(text=iface_("Original frame range: %d-%d (%d)") % (sta, end, end - sta + 1), translate=False)

            row = layout.row()
            row.prop(strip, "use_sequence", text="Sequence")
            sub = row.row()
            sub.enabled = not strip.use_sequence
            sub.prop(strip, "use_grease_pencil", text="Grease Pencil")

            if not strip.use_sequence:                
                split = layout.split(percentage=0.5)
                split.label(text="Camera Override:")
                split.template_ID(strip, "scene_camera")


class SEQUENCER_PT_mask(SequencerButtonsPanel, Panel):
    bl_label = "Mask"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return (strip.type == 'MASK')

    def draw(self, context):
        layout = self.layout

        strip = act_strip(context)

        layout.template_ID(strip, "mask")

        mask = strip.mask

        if mask:
            sta = mask.frame_start
            end = mask.frame_end
            layout.label(text=iface_("Original frame range: %d-%d (%d)") % (sta, end, end - sta + 1), translate=False)


            
            
        
class SEQUENCER_PT_filter(SequencerButtonsPanel, Panel):
    bl_label = "Filter"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return strip.type in {'MOVIE', 'IMAGE', 'SCENE', 'MOVIECLIP', 'MASK',
                              'META', 'ADD', 'SUBTRACT', 'ALPHA_OVER',
                              'ALPHA_UNDER', 'CROSS', 'GAMMA_CROSS', 'MULTIPLY',
                              'OVER_DROP', 'WIPE', 'GLOW', 'TRANSFORM',
                              'MULTICAM', 'SPEED', 'ADJUSTMENT','GAUSSIAN_BLUR'}

    def draw(self, context):
        layout = self.layout

        strip = act_strip(context)
        
        if strip.type == 'MOVIECLIP':
            row = layout.row()
            row.prop(strip, "stabilize2d", text="Stabilize")
            row.prop(strip, "undistort", text="Undistort")
          
        row = layout.row()
        col = row.column(align=True)
        col.prop(strip, "strobe", text="Strobe")
        col = row.column(align=True)
        col.prop(strip, "use_reverse_frames", text="Backwards")
        
        row = layout.row()
        col = row.column(align=True)
        col.prop(strip, "color_saturation", text="Saturation")
        col.prop(strip, "color_multiply", text="Multiply")
        col = row.column(align=True)
        col.enabled = (strip.type in ["MOVIE", "IMAGE", "MOVIECLIP", "ADJUSTMENT", "MULTICAM"])
        col.prop(strip, "use_float")
        col.prop(strip, "use_deinterlace")

        
        
        
class SEQUENCER_PT_proxy(SequencerButtonsPanel, Panel):
    bl_label = "Proxy / Timecode"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return strip.type in {'MOVIE', 'IMAGE', 'SCENE', 'META', 'MULTICAM'}

    def draw_header(self, context):
        strip = act_strip(context)

        self.layout.prop(strip, "use_proxy", text="")
        
    def draw(self, context):
        layout = self.layout

        sequencer = context.scene.sequence_editor
        strip = act_strip(context)
        
        
        if strip.use_proxy and strip.proxy:
            proxy = strip.proxy
            
            col = layout.column(align=True)
            
            col.label(text="Resolutions:")
            row = col.row(align=True)
            row.prop(proxy, "build_25", toggle=True)
            row.prop(proxy, "build_50", toggle=True)
            row.prop(proxy, "build_75", toggle=True)
            row.prop(proxy, "build_100", toggle=True)
            
            split = layout.split(percentage=0.25, align=True)
            split.prop(proxy, "use_proxy_custom_file", text="File", toggle=True)
            sub = split.row(align=True)
            sub.enabled = proxy.use_proxy_custom_file
            sub.prop(proxy, "filepath", text="")
            
            if not proxy.use_proxy_custom_file:
                col = layout.column()
                split = col.split(percentage=0.33)
                split.label(text="Storage:")
                row = split.row()
                row.prop(sequencer, "proxy_storage", expand=True)
                
                col = layout.column()
                if sequencer.proxy_storage == 'PROJECT':
                    col.prop(sequencer, "proxy_dir", text="")
                else:
                    split = col.split(percentage=0.25, align=True)
                    split.prop(proxy, "use_proxy_custom_directory", text="Custom", toggle=True)
                    sub = split.row(align=True)
                    sub.enabled = proxy.use_proxy_custom_directory
                    sub.prop(proxy, "directory", text="")
                        
                col = layout.column()
                row = col.row()
                row.prop(proxy, "quality", slider=True)
                row.prop(proxy, "use_overwrite")
                
                if strip.type == 'MOVIE':
                    col = layout.column()
                    split = col.split(percentage=0.33, align=True)
                    split.label(text="Timecode:")
                    split.prop(proxy, "timecode", text="")

        col = layout.column()
        col.label(text="Selected Strips:")
        row = col.row()
        row.operator("sequencer.enable_proxies", text="Set Proxy", icon="SEQ_SEQUENCER")
        row.operator("sequencer.rebuild_proxy", text="Build Proxy", icon="RENDER_STILL")
  
            
            
class SEQUENCER_PT_preview(SequencerButtonsPanel_Output, Panel):
    bl_label = "Scene Preview/Render"
    bl_space_type = 'SEQUENCE_EDITOR'
    bl_region_type = 'UI'

    def draw(self, context):
        layout = self.layout

        render = context.scene.render

        col = layout.column()
        col.prop(render, "use_sequencer_gl_preview", text="OpenGL Preview")
        col = layout.column()
        #col.active = render.use_sequencer_gl_preview
        col.prop(render, "sequencer_gl_preview", text="")

        row = col.row()
        row.active = render.sequencer_gl_preview == 'SOLID'
        row.prop(render, "use_sequencer_gl_textured_solid")


class SEQUENCER_PT_view(SequencerButtonsPanel_Output, Panel):
    bl_label = "View Settings"

    def draw(self, context):
        layout = self.layout

        st = context.space_data

        col = layout.column()
        if st.display_mode == 'IMAGE':
            col.prop(st, "draw_overexposed")
            col.separator()

        elif st.display_mode == 'WAVEFORM':
            col.prop(st, "show_separate_color")

        col = layout.column()
        col.separator()
        col.prop(st, "proxy_render_size")


class SEQUENCER_PT_view_safe_areas(SequencerButtonsPanel_Output, Panel):
    bl_label = "Safe Areas"
    bl_options = {'DEFAULT_CLOSED'}

    @classmethod
    def poll(cls, context):
        st = context.space_data
        is_preview = st.view_type in {'PREVIEW', 'SEQUENCER_PREVIEW'}
        return is_preview and (st.display_mode == 'IMAGE')

    def draw_header(self, context):
        st = context.space_data

        self.layout.prop(st, "show_safe_areas", text="")

    def draw(self, context):
        from bl_ui.properties_data_camera import draw_display_safe_settings

        layout = self.layout
        st = context.space_data
        safe_data = context.scene.safe_areas

        draw_display_safe_settings(layout, safe_data, st)


class SEQUENCER_PT_modifiers(SequencerButtonsPanel, Panel):
    bl_label = "Modifiers"

    @classmethod
    def poll(cls, context):
        if not cls.has_sequencer(context):
            return False

        strip = act_strip(context)
        if not strip:
            return False

        return not strip.type in {'SOUND'}
    
    def draw(self, context):
        layout = self.layout

        strip = act_strip(context)
        sequencer = context.scene.sequence_editor

        layout.operator_menu_enum("sequencer.strip_modifier_add", "type")
        row = layout.row()
        row.operator("sequencer.strip_modifier_copy", text="Copy to Selected")
        row.prop(strip, "use_linear_modifiers", text="Linear Colorspace")

        for mod in strip.modifiers:
            box = layout.box()

            row = box.row()
            row.prop(mod, "show_expanded", text="", emboss=False)
            sub = row.row(align=True)
            sub.prop(mod, "name", text="")
            sub.prop(mod, "mute", text="")

            sub = row.row(align=True)
            props = sub.operator("sequencer.strip_modifier_move", text="", icon='TRIA_UP')
            props.name = mod.name
            props.direction = 'UP'
            props = sub.operator("sequencer.strip_modifier_move", text="", icon='TRIA_DOWN')
            props.name = mod.name
            props.direction = 'DOWN'
            
            row.operator("sequencer.strip_modifier_remove", text="", icon='X', emboss=False).name = mod.name

            if mod.show_expanded:
                box.enabled = not mod.mute
                
                if mod.type == 'COLOR_BALANCE':
                    draw_color_balance(box, mod.color_balance)
                    split = box.split(percentage=0.25)
                    split.label(text="Multiply:")
                    split.prop(mod, "color_multiply", text="")
                    
                elif mod.type == 'CURVES':
                    box.template_curve_mapping(mod, "curve_mapping", type='COLOR')
                    
                elif mod.type == 'HUE_CORRECT':
                    box.template_curve_mapping(mod, "curve_mapping", type='HUE')
                    
                elif mod.type == 'BRIGHT_CONTRAST':
                    row = box.row()
                    col = row.column()
                    col.label(text="Brigh:")
                    col.prop(mod, "bright", text="")
                    col = row.column()
                    col.label(text="Contrast:")
                    col.prop(mod, "contrast", text="")
                    
                elif mod.type == 'WHITE_BALANCE':
                    split = box.split(percentage=0.45)
                    col = split.column()
                    col.label(text="White:")
                    col.prop(mod, "white_value", text="")
                    split.template_color_picker(mod, "white_value", value_slider=True, lock=True, lock_luminosity=True, cubic=True)
                    
                elif mod.type == 'TONEMAP':
                    row = box.row()
                    row.prop(mod, "tonemap_type", expand=True)
                    row = box.row()
                    col = row.column(align=True)
                    if mod.tonemap_type == 'RD_PHOTORECEPTOR':
                        col.prop(mod, "intensity", slider=True)
                        col.prop(mod, "contrast", slider=True)
                        col = row.column(align=True)
                        col.prop(mod, "adaptation", slider=True)
                        col.prop(mod, "correction", slider=True)
                    elif mod.tonemap_type == 'RH_SIMPLE':
                        col.prop(mod, "key", slider=True)
                        col.prop(mod, "offset", slider=True)
                        col = row.column(align=True)
                        col.prop(mod, "gamma", slider=True)
            
                col = box.column()
                split = col.split(percentage=0.25)
                split.label(text="Mask:")
                row = split.row()
                row.prop(mod, "input_mask_type", expand=True)
                
                if mod.input_mask_type == 'STRIP':
                    sequences_object = sequencer
                    if sequencer.meta_stack:
                        sequences_object = sequencer.meta_stack[-1]
                    split = col.split(percentage=0.25)
                    split.label(text="Strip")
                    split.prop_search(mod, "input_mask_strip", sequences_object, "sequences", text="")
                else:
                    split = col.split(align=True)
                    split.prop(mod, "input_mask_id", text="")
                    row = split.row(align=True)
                    row.prop(mod, "mask_time", expand=True)

class SEQUENCER_PT_grease_pencil(GreasePencilDataPanel, SequencerButtonsPanel_Output, Panel):
    bl_space_type = 'SEQUENCE_EDITOR'
    bl_region_type = 'UI'

    # NOTE: this is just a wrapper around the generic GP Panel
    # But, it should only show up when there are images in the preview region


class SEQUENCER_PT_grease_pencil_tools(GreasePencilToolsPanel, SequencerButtonsPanel_Output, Panel):
    bl_space_type = 'SEQUENCE_EDITOR'
    bl_region_type = 'UI'

    # NOTE: this is just a wrapper around the generic GP tools panel
    # It contains access to some essential tools usually found only in
    # toolbar, which doesn't exist here...


class SEQUENCER_PT_custom_props(SequencerButtonsPanel, PropertyPanel, Panel):
    COMPAT_ENGINES = {'BLENDER_RENDER', 'BLENDER_GAME'}
    _context_path = "scene.sequence_editor.active_strip"
    _property_type = (bpy.types.Sequence,)



if __name__ == "__main__":  # only for live edit.
    bpy.utils.register_module(__name__)
